@@ -21,6 +21,7 @@ import ssl
 import sys
 import urllib
 import warnings
+import io
 
 import tempfile as tf
 from time import sleep
@@ -38,13 +39,12 @@ class SASconfigHTTP:
    This object is not intended to be used directly. Instantiate a SASsession object instead 
    '''
    def __init__(self, session, **kwargs):
-      self._kernel  = kwargs.get('kernel', None)   
-      self._token   = None
-
+      self._kernel   = kwargs.get('kernel', None)   
       SAScfg         = session._sb.sascfg.SAScfg
       self.name      = session._sb.sascfg.name
       cfg            = getattr(SAScfg, self.name)
 
+      self._token    = cfg.get('authtoken', None)
       self.url       = cfg.get('url', '')
       self.ip        = cfg.get('ip', '')
       self.port      = cfg.get('port', None)
@@ -56,6 +56,9 @@ class SASconfigHTTP:
       self.timeout   = cfg.get('timeout', None)
       user           = cfg.get('user', '')
       pw             = cfg.get('pw', '')
+      client_id      = cfg.get('client_id', 'SASPy')
+      client_secret  = cfg.get('client_secret', '')
+      authcode       = cfg.get('authcode', None)
       self.encoding  = cfg.get('encoding', '')
       self.authkey   = cfg.get('authkey', '')
       self._prompt   = session._sb.sascfg._prompt
@@ -148,8 +151,36 @@ class SASconfigHTTP:
       if not self.encoding or self.encoding != 'utf_8':
          self.encoding = 'utf_8'
 
+      inautht = kwargs.get('authtoken', None)
+      if inautht is not None:
+         if lock and self._token:
+            print("Parameter 'authtoken' passed to SAS_session was ignored due to configuration restriction.")
+         else:
+            self._token = inautht
+
+      inauthc = kwargs.get('authcode', None)
+      if inauthc is not None:
+         if lock and authcode:
+            print("Parameter 'authcode' passed to SAS_session was ignored due to configuration restriction.")
+         else:
+            authcode = inauthc
+
+      incis = kwargs.get('client_secret', None)
+      if incis is not None:
+         if lock and client_secret:
+            print("Parameter 'client_secret' passed to SAS_session was ignored due to configuration restriction.")
+         else:
+            client_secret = incis
+
+      incid = kwargs.get('client_id', None)
+      if incid is not None:
+         if lock and client_id:
+            print("Parameter 'client_id' passed to SAS_session was ignored due to configuration restriction.")
+         else:
+            client_id = incid
+
       inlrecl = kwargs.get('lrecl', None)
-      if inlrecl:
+      if inlrecl is not None:
          if lock and self.lrecl:
             print("Parameter 'lrecl' passed to SAS_session was ignored due to configuration restriction.")
          else:
@@ -190,76 +221,94 @@ class SASconfigHTTP:
          else:
             port = 80
 
-      found = False
-      if self.authkey:
-         if os.name == 'nt': 
-            pwf = os.path.expanduser('~')+os.sep+'_authinfo'
-         else:
-            pwf = os.path.expanduser('~')+os.sep+'.authinfo'
-         try:
-            fid = open(pwf, mode='r')
-            for line in fid:
-               if line.startswith(self.authkey): 
-                  user = line.partition('user')[2].lstrip().partition(' ')[0].partition('\n')[0]
-                  pw   = line.partition('password')[2].lstrip().partition(' ')[0].partition('\n')[0]
-                  found = True
-                  break
-            fid.close()
-         except OSError as e:
-            print('Error trying to read authinfo file:'+pwf+'\n'+str(e))
-            pass
-         except:
-            pass
+      if not self._token and not authcode:
+         found = False
+         if self.authkey:
+            if os.name == 'nt': 
+               pwf = os.path.expanduser('~')+os.sep+'_authinfo'
+            else:
+               pwf = os.path.expanduser('~')+os.sep+'.authinfo'
+            try:
+               fid = open(pwf, mode='r')
+               for line in fid:
+                  if line.startswith(self.authkey): 
+                     user = line.partition('user')[2].lstrip().partition(' ')[0].partition('\n')[0]
+                     pw   = line.partition('password')[2].lstrip().partition(' ')[0].partition('\n')[0]
+                     found = True
+                     break
+               fid.close()
+            except OSError as e:
+               print('Error trying to read authinfo file:'+pwf+'\n'+str(e))
+               pass
+            except:
+               pass
+      
+            if not found:
+               print('Did not find key '+self.authkey+' in authinfo file:'+pwf+'\n')
    
-         if not found:
-            print('Did not find key '+self.authkey+' in authinfo file:'+pwf+'\n')
+         inuser = kwargs.get('user', '')              
+         if len(inuser) > 0:
+            if lock and len(user):
+               print("Parameter 'user' passed to SAS_session was ignored due to configuration restriction.")
+            else:
+               user = inuser
+   
+         inpw = kwargs.get('pw', '')                  
+         if len(inpw) > 0:
+            if lock and len(pw):
+               print("Parameter 'pw' passed to SAS_session was ignored due to configuration restriction.")
+            else:
+               pw = inpw
 
-      inuser = kwargs.get('user', '')              
-      if len(inuser) > 0:
-         if lock and len(user):
-            print("Parameter 'user' passed to SAS_session was ignored due to configuration restriction.")
+         code_pw = ''
+         if len(user) == 0:
+            msg  = "To connect to Viya you need either an authcode or a userid/pw. Neither were provided.\n"
+            msg += "Please enter which one you want to enter next. Type one of these now: [authcode | userid]" 
+            while code_pw.lower() not in ['userid','authcode']:
+               code_pw = self._prompt(msg)
+               if code_pw is None:
+                  self._token = None
+                  raise RuntimeError("Neither authcode nor userid provided.") 
+
+         if code_pw.lower() == 'authcode':
+            authcode = self._prompt("Please enter authcode: ")
+            if authcode is None:
+               self._token = None
+               raise RuntimeError("No authcode provided.") 
          else:
-            user = inuser
-
-      inpw = kwargs.get('pw', '')                  
-      if len(inpw) > 0:
-         if lock and len(pw):
-            print("Parameter 'pw' passed to SAS_session was ignored due to configuration restriction.")
-         else:
-            pw = inpw
-
-      while len(user) == 0:
-         user = self._prompt("Please enter userid: ")
-         if user is None:
-            self._token = None
-            raise RuntimeError("No userid provided.") 
-
-      while len(pw) == 0:
-         pw = self._prompt("Please enter password: ", pw = True)
-         if pw is None:
-            self._token = None
-            raise RuntimeError("No password provided.") 
+            while len(user) == 0:
+               user = self._prompt("Please enter userid: ")
+               if user is None:
+                  self._token = None
+                  raise RuntimeError("No userid provided.") 
+        
+            while len(pw) == 0:
+               pw = self._prompt("Please enter password: ", pw = True)
+               if pw is None:
+                  self._token = None
+                  raise RuntimeError("No password provided.") 
 
       if self.ssl:
          if self.verify:
             # handle having self signed certificate default on Viya w/out copies on client; still ssl, just not verifyable
             try:
                self.HTTPConn = hc.HTTPSConnection(self.ip, self.port, timeout=self.timeout)
-               self._token = self._authenticate(user, pw)
+               if not self._token:
+                  self._token = self._authenticate(user, pw, authcode, client_id, client_secret)
             except ssl.SSLError as e:
                print("SSL certificate verification failed, creating an unverified SSL connection. Error was:"+str(e))
                self.HTTPConn = hc.HTTPSConnection(self.ip, self.port, timeout=self.timeout, context=ssl._create_unverified_context())
                print("You can set 'verify=False' to get rid of this message ")
-               self._token   = self._authenticate(user, pw)
+               if not self._token:
+                  self._token   = self._authenticate(user, pw, authcode, client_id, client_secret)
          else:
             self.HTTPConn = hc.HTTPSConnection(self.ip, self.port, timeout=self.timeout, context=ssl._create_unverified_context())
-            self._token = self._authenticate(user, pw)
+            if not self._token:
+               self._token = self._authenticate(user, pw, authcode, client_id, client_secret)
       else:
          self.HTTPConn = hc.HTTPConnection(self.ip, self.port, timeout=self.timeout)
-         self._token   = self._authenticate(user, pw)
-
-      # get AuthToken
-      #self._token = self._authenticate(user, pw)
+         if not self._token:
+            self._token   = self._authenticate(user, pw, authcode, client_id, client_secret)
 
       if not self._token:
          print("Could not acquire an Authentication Token")
@@ -338,18 +387,25 @@ class SASconfigHTTP:
 
       return
 
-   def _authenticate(self, user, pw):
+   def _authenticate(self, user, pw, authcode, client_id, client_secret):
       #import pdb; pdb.set_trace()
-      uuser = urllib.parse.quote(user)
-      upw   = urllib.parse.quote(pw)
+      if authcode:
+         uauthcode      = urllib.parse.quote(authcode)
+         uclient_id     = urllib.parse.quote(client_id)
+         uclient_secret = urllib.parse.quote(client_secret)
+         d1 = ("grant_type=authorization_code&code="+uauthcode+"&client_id="+uclient_id+"&client_secret="+uclient_secret).encode(self.encoding)
+         headers={"Accept":"application/vnd.sas.compute.session+json","Content-Type":"application/x-www-form-urlencoded"} 
+      else:
+         uuser = urllib.parse.quote(user)
+         upw   = urllib.parse.quote(pw)
+         d1 = ("grant_type=password&username="+uuser+"&password="+upw).encode(self.encoding)
+         basic = base64.encodebytes("sas.tkmtrb:".encode(self.encoding))
+         authheader = '%s' % basic.splitlines()[0].decode(self.encoding)
+         headers={"Accept":"application/vnd.sas.compute.session+json","Content-Type":"application/x-www-form-urlencoded",
+                  "Authorization":"Basic "+authheader}
       
       # POST AuthToken
       conn = self.HTTPConn; conn.connect()
-      d1 = ("grant_type=password&username="+uuser+"&password="+upw).encode(self.encoding)
-      basic = base64.encodebytes("sas.tkmtrb:".encode(self.encoding))
-      authheader = '%s' % basic.splitlines()[0].decode(self.encoding)
-      headers={"Accept":"application/vnd.sas.compute.session+json","Content-Type":"application/x-www-form-urlencoded",
-               "Authorization":"Basic "+authheader}
       try:
          conn.request('POST', "/SASLogon/oauth/token", body=d1, headers=headers)
          req = conn.getresponse()
@@ -581,20 +637,18 @@ class SASsessionHTTP():
 
       # GET Log
       if jobid:
-         lines = 9999999 #jobid.get('logInfo').get('lineCount')
          for ld in jobid.get('links'):
             if ld.get('method') == 'GET' and ld.get('rel') == 'log':
                uri = ld.get('uri')
                break
       else:
-         lines = 9999999 #self._session.get('logStatistics').get('lineCount')
          uri   = self._uri_log
 
       while True:
          # GET Log
          conn = self.sascfg.HTTPConn; conn.connect()
          headers={"Accept":"application/vnd.sas.collection+json", "Authorization":"Bearer "+self.sascfg._token}
-         conn.request('GET', uri+"?start="+str(start)+"&limit="+str(lines+1), headers=headers)
+         conn.request('GET', uri+"?start="+str(start)+"&limit="+str(start+1000), headers=headers)
          req = conn.getresponse()
          status = req.status
          resp = req.read()
@@ -611,9 +665,8 @@ class SASsessionHTTP():
             break
          start += lines
 
-         for i in range(len(log)):
-             line = dict(log[i]).get('line')
-             logr += line+'\n'
+         for line in log:
+             logr += line.get('line')+'\n'
 
       if jobid != None:   
          self._log += logr.replace(chr(12), chr(10))
@@ -679,15 +732,13 @@ class SASsessionHTTP():
             if ld.get('method') == 'GET' and ld.get('rel') == 'listing':
                uri = ld.get('uri')
                break
-         lines = 9999999 #jobid.get('listInfo').get('lineCount')
       else:
-         lines = 9999999 #self._session.get('listingStatistics').get('lineCount')
          uri   = self._uri_lst
 
       while True:
          conn = self.sascfg.HTTPConn; conn.connect()
          headers={"Accept":"application/vnd.sas.collection+json", "Authorization":"Bearer "+self.sascfg._token}
-         conn.request('GET', uri+"?start="+str(start)+"&limit="+str(lines+1), headers=headers)
+         conn.request('GET', uri+"?start="+str(start)+"&limit="+str(start+1000), headers=headers)
          req = conn.getresponse()
          status = req.status
          resp = req.read()
@@ -704,9 +755,8 @@ class SASsessionHTTP():
             break
          start += lines
 
-         for i in range(len(lst)):
-             line = dict(lst[i]).get('line')
-             lstr += line+'\n'
+         for line in lst:
+             lstr += line.get('line')+'\n'
 
       return lstr
 
@@ -1397,7 +1447,6 @@ class SASsessionHTTP():
       table   - the name of the SAS Data Set you want to export to a Pandas Data Frame
       libref  - the libref for the SAS Data Set.
       dsopts  - data set options for the input SAS Data Set
-      Only for DISK version:
       rowsep  - the row seperator character to use; defaults to '\x01'
       colsep  - the column seperator character to use; defaults to '\x02'
       rowrep  - the char to convert to for any embedded rowsep chars, defaults to  ' '
@@ -1408,186 +1457,22 @@ class SASsessionHTTP():
       method = kwargs.pop('method', None)
       if   method and method.lower() == 'csv':
          return self.sasdata2dataframeCSV(table, libref, dsopts, **kwargs)
-      elif method and method.lower() == 'disk':
+      #elif method and method.lower() == 'disk':
+      else:
          return self.sasdata2dataframeDISK(table, libref, dsopts, rowsep, colsep,
                                            rowrep, colrep, **kwargs)
 
-      my_fmts = kwargs.pop('my_fmts', False)
-      k_dts   = kwargs.pop('dtype',   None)
-      if self.sascfg.verbose:
-         if my_fmts != False:
-            print("'my_fmts=' is only used with the CSV or DISK version of this method. option ignored.")
-         if k_dts is not None:
-            print("'dtype=' is only used with the CSV or DISK version of this method. option ignored.")
-
-      if libref:
-         tabname = libref+".'"+table.strip()+"'n "
-      else:
-         tabname = "'"+table.strip()+"'n "
-
-      code  = "data work.sasdata2dataframe / view=work.sasdata2dataframe; set "+tabname+self._sb._dsopts(dsopts)+";run;\n"
-
-      ll = self.submit(code, "text")
-
-      ##GET Data Table Info
-      #conn = self.sascfg.HTTPConn; conn.connect()
-      #headers={"Accept":"application/vnd.sas.compute.data.table+json", "Authorization":"Bearer "+self.sascfg._token}
-      #conn.request('GET', "/compute/sessions/"+self.pid+"/data/work/sasdata2dataframe", headers=headers)
-      #req = conn.getresponse()
-      #status = req.status
-      #conn.close()
-
-      #resp = req.read()
-      #js = json.loads(resp.decode(self.sascfg.encoding))
-
-      conn = self.sascfg.HTTPConn; conn.connect()
-      headers={"Accept":"application/vnd.sas.collection+json", "Authorization":"Bearer "+self.sascfg._token}
-      conn.request('GET', "/compute/sessions/"+self.pid+"/data/work/sasdata2dataframe/columns?start=0&limit=9999999", headers=headers)
-      req = conn.getresponse()
-      status = req.status
-      resp = req.read()
-      conn.close()
-
-      js = json.loads(resp.decode(self.sascfg.encoding))
-
-      varlist = []
-      vartype = []
-      nvars = js.get('count')
-      lst = js.get('items')
-      for i in range(len(lst)):
-         varlist.append(lst[i].get('name'))
-         vartype.append(lst[i].get('type'))
-
-      topts = dict(dsopts)
-      topts.pop('firstobs', None)
-      topts.pop('obs', None)
-   
-      code  = "proc delete data=work.sasdata2dataframe(memtype=view);run;\n"
-      code += "data work._n_u_l_l_;output;run;\n"
-      code += "data _null_; set work._n_u_l_l_ "+tabname+self._sb._dsopts(topts)+";put 'FMT_CATS=';\n"
-   
-      for i in range(nvars):
-         code += "_tom = vformatn('"+varlist[i]+"'n);put _tom;\n"
-      code += "stop;\nrun;\nproc delete data=work._n_u_l_l_;run;"
-   
-      ll = self.submit(code, "text")
-
-      l2 = ll['LOG'].rpartition("FMT_CATS=")
-      l2 = l2[2].partition("\n")              
-      varcat = l2[2].split("\n", nvars)
-      del varcat[nvars]
-
-      code  = "data work.sasdata2dataframe / view=work.sasdata2dataframe; set "+tabname+self._sb._dsopts(dsopts)+";\n"
-      for i in range(nvars):
-         if vartype[i] == 'FLOAT':
-            code += "format '"+varlist[i]+"'n "
-            if varcat[i] in self._sb.sas_date_fmts:
-               code += 'E8601DA10.'
-            else:
-               if varcat[i] in self._sb.sas_time_fmts:
-                  code += 'E8601TM15.6'
-               else:
-                  if varcat[i] in self._sb.sas_datetime_fmts:
-                     code += 'E8601DT26.6'
-                  else:
-                     code += 'best32.'
-            code += '; '
-            if i % 10 == 0:
-               code +='\n'
-      code += ";run;\n"
-      ll = self.submit(code, "text")
-
-      conn = self.sascfg.HTTPConn; conn.connect()
-      headers={"Accept":"application/vnd.sas.collection+json", "Authorization":"Bearer "+self.sascfg._token}
-      uri = "/compute/sessions/"+self.pid+"/data/work/sasdata2dataframe/rows"
-
-      r     = []
-      df    = None
-      trows = kwargs.get('trows', None)
-      if not trows:
-         trows = 100000
-
-      while True:
-         conn.request('GET', uri, headers=headers)
-         req    = conn.getresponse()
-         status = req.status
-         resp   = req.read()
-         conn.close()
-   
-         js = json.loads(resp.decode(self.sascfg.encoding))
-   
-         lst = js.get('items')
-
-         if not lst:
-            break
-
-         for i in range(len(lst)):
-            r.append(lst[i]['cells'])
-
-         if len(r) > trows:   
-            tdf = pd.DataFrame.from_records(r, columns=varlist)
-                       
-            for i in range(nvars):
-               if vartype[i] == 'FLOAT':
-                  if varcat[i] not in self._sb.sas_date_fmts + self._sb.sas_time_fmts + self._sb.sas_datetime_fmts:
-                     if tdf.dtypes[tdf.columns[i]].kind not in ('f','u','i','b','B','c','?'):
-                        tdf[varlist[i]] = pd.to_numeric(tdf[varlist[i]], errors='coerce') 
-                  else:
-                     if tdf.dtypes[tdf.columns[i]].kind not in ('M'):
-                        tdf[varlist[i]] = pd.to_datetime(tdf[varlist[i]], errors='coerce') 
-               else:
-                  tdf[varlist[i]] = tdf[varlist[i]].apply(str.strip)
-                  tdf[varlist[i]].replace('', np.NaN, True)
-                                          
-            if df is not None:
-               df = df.append(tdf, ignore_index=True)
-            else:
-               df = tdf
-            r = []
-               
-         uri = None
-         for ld in js.get('links'):
-            if ld.get('method') == 'GET' and ld.get('rel') == 'next':
-               uri = ld.get('uri')
-               break
-
-         if not uri:
-            break
-
-      if len(r) > 0:   
-         tdf = pd.DataFrame.from_records(r, columns=varlist)
-
-         for i in range(nvars):
-            if vartype[i] == 'FLOAT':
-               if varcat[i] not in self._sb.sas_date_fmts + self._sb.sas_time_fmts + self._sb.sas_datetime_fmts:
-                  if tdf.dtypes[tdf.columns[i]].kind not in ('f','u','i','b','B','c','?'):
-                     tdf[varlist[i]] = pd.to_numeric(tdf[varlist[i]], errors='coerce') 
-               else:
-                  tdf[varlist[i]] = pd.to_datetime(tdf[varlist[i]], errors='coerce') 
-            else:
-               tdf[varlist[i]] = tdf[varlist[i]].apply(str.strip)
-               tdf[varlist[i]].replace('', np.NaN, True)
-
-         if df is not None:
-            df = df.append(tdf, ignore_index=True)
-         else:
-            df = tdf
-
-      ll = self.submit("proc delete data=work.sasdata2dataframe(memtype=view);run;", "text")
-
-      return df
-
 
    def sasdata2dataframeCSV(self, table: str, libref: str ='', dsopts: dict =None, opts: dict = None, 
-                            tempfile: str=None, tempkeep: bool=False, **kwargs) -> '<Pandas Data Frame object>':
+                            **kwargs) -> '<Pandas Data Frame object>':
       '''
       This method exports the SAS Data Set to a Pandas Data Frame, returning the Data Frame object.
       table    - the name of the SAS Data Set you want to export to a Pandas Data Frame
       libref   - the libref for the SAS Data Set.
       dsopts   - data set options for the input SAS Data Set
       opts     - a dictionary containing any of the following Proc Export options(delimiter, putnames)
-      tempfile - file to use to store CSV, else temporary file will be used.
-      tempkeep - if you specify your own file to use with tempfile=, this controls whether it's cleaned up after using it
+      tempfile - DEPRECATED
+      tempkeep - DEPRECATED
 
       These two options are for advanced usage. They override how saspy imports data. For more info
       see https://sassoftware.github.io/saspy/advanced-topics.html#advanced-sd2df-and-df2sd-techniques
@@ -1595,6 +1480,9 @@ class SASsessionHTTP():
       dtype   - this is the parameter to Pandas read_csv, overriding what saspy generates and uses
       my_fmts - bool: if True, overrides the formats saspy would use, using those on the data set or in dsopts=
       '''
+      tmp = kwargs.pop('tempfile', None)
+      tmp = kwargs.pop('tempkeep', None)
+
       dsopts = dsopts if dsopts is not None else {}
       opts   = opts   if   opts is not None else {}
 
@@ -1603,14 +1491,6 @@ class SASsessionHTTP():
       else:
          tabname = "'"+table.strip()+"'n "
 
-      tmpdir  = None
-
-      if tempfile is None:
-         tmpdir = tf.TemporaryDirectory()
-         tmpcsv = tmpdir.name+os.sep+"tomodsx"
-      else:
-         tmpcsv  = tempfile
-
       code  = "data work.sasdata2dataframe / view=work.sasdata2dataframe; set "+tabname+self._sb._dsopts(dsopts)+";run;\n"
 
       ll = self.submit(code, "text")
@@ -1676,16 +1556,17 @@ class SASsessionHTTP():
       if not my_fmts:
          for i in range(nvars):
             if vartype[i] == 'FLOAT':
+               code += "'"+varlist[i]+"'n "
                if varcat[i] in self._sb.sas_date_fmts:
-                  code += "'"+varlist[i]+"'n E8601DA10. "
+                  code += 'E8601DA10. '
                else:
                   if varcat[i] in self._sb.sas_time_fmts:
-                     code += "'"+varlist[i]+"'n E8601TM15.6 "
+                     code += 'E8601TM15.6 '
                   else:
                      if varcat[i] in self._sb.sas_datetime_fmts:
-                        code += "'"+varlist[i]+"'n E8601DT26.6 "
+                        code += 'E8601DT26.6 '
                      else:
-                        code += "'"+varlist[i]+"'n best32. "
+                        code += 'best32. '
       code += ";run;\n"
       ll = self.submit(code, "text")
 
@@ -1708,16 +1589,26 @@ class SASsessionHTTP():
       code += "proc delete data=work.sasdata2dataframe(memtype=view);run;\n"
 
       ll = self.submit(code, 'text')
+      logf  = ll['LOG']
 
-      ll = self.download(tmpcsv, self._sb.workpath+"_tomodsx")
+      code = "filename _sp_updn '"+self._sb.workpath+"_tomodsx' recfm=F encoding=binary lrecl=4096;"
 
-      df = pd.read_csv(tmpcsv, index_col=idx_col, engine=eng, dtype=dts, **kwargs)
+      ll = self.submit(code, "text")
+      logf += ll['LOG']
 
-      if tmpdir:
-         tmpdir.cleanup()
-      else:
-         if not tempkeep:
-            os.remove(tmpcsv)
+      # GET data
+      conn = self.sascfg.HTTPConn; conn.connect()
+      headers={"Accept":"*/*","Content-Type":"application/octet-stream",
+               "Authorization":"Bearer "+self.sascfg._token}
+      conn.request('GET', self._uri_files+"/_sp_updn/content", headers=headers)
+      req = conn.getresponse()
+      status = req.status
+
+      sockout = _read_sock(req=req)
+
+      df = pd.read_csv(sockout, index_col=idx_col, encoding='utf8', engine=eng, dtype=dts, **kwargs)
+
+      conn.close()
 
       if k_dts is None:  # don't override these if user provided their own dtypes
          for i in range(nvars):
@@ -1725,12 +1616,14 @@ class SASsessionHTTP():
                if varcat[i] in self._sb.sas_date_fmts + self._sb.sas_time_fmts + self._sb.sas_datetime_fmts:
                   df[varlist[i]] = pd.to_datetime(df[varlist[i]], errors='coerce')
 
+      ll = self.submit("filename _sp_updn;", 'text')
+      logf += ll['LOG']
+
       return df
 
    def sasdata2dataframeDISK(self, table: str, libref: str ='', dsopts: dict = None,
                              rowsep: str = '\x01', colsep: str = '\x02',
-                             rowrep: str = ' ',    colrep: str = ' ', tempfile: str=None, 
-                             tempkeep: bool=False, **kwargs) -> '<Pandas Data Frame object>':
+                             rowrep: str = ' ',    colrep: str = ' ', **kwargs) -> '<Pandas Data Frame object>':
       '''
       This method exports the SAS Data Set to a Pandas Data Frame, returning the Data Frame object.
       table    - the name of the SAS Data Set you want to export to a Pandas Data Frame
@@ -1740,8 +1633,8 @@ class SASsessionHTTP():
       colsep   - the column seperator character to use; defaults to '\x02'
       rowrep  - the char to convert to for any embedded rowsep chars, defaults to  ' '
       colrep  - the char to convert to for any embedded colsep chars, defaults to  ' '
-      tempfile - file to use to store CSV, else temporary file will be used.
-      tempkeep - if you specify your own file to use with tempfile=, this controls whether it's cleaned up after using it
+      tempfile - DEPRECATED
+      tempkeep - DEPRECATED
 
       These two options are for advanced usage. They override how saspy imports data. For more info
       see https://sassoftware.github.io/saspy/advanced-topics.html#advanced-sd2df-and-df2sd-techniques
@@ -1749,6 +1642,9 @@ class SASsessionHTTP():
       dtype   - this is the parameter to Pandas read_csv, overriding what saspy generates and uses
       my_fmts - bool: if True, overrides the formats saspy would use, using those on the data set or in dsopts=
       '''
+      tmp = kwargs.pop('tempfile', None)
+      tmp = kwargs.pop('tempkeep', None)
+
       dsopts = dsopts if dsopts is not None else {}
 
       if libref:
@@ -1756,14 +1652,6 @@ class SASsessionHTTP():
       else:
          tabname = "'"+table.strip()+"'n "
 
-      tmpdir  = None
-
-      if tempfile is None:
-         tmpdir = tf.TemporaryDirectory()
-         tmpcsv = tmpdir.name+os.sep+"tomodsx"
-      else:
-         tmpcsv  = tempfile
-
       code  = "data work.sasdata2dataframe / view=work.sasdata2dataframe; set "+tabname+self._sb._dsopts(dsopts)+";run;\n"
 
       ll = self.submit(code, "text")
@@ -1827,7 +1715,7 @@ class SASsessionHTTP():
          print("my_fmts option only valid when dtype= is specified. Ignoring and using necessary formatting for data transfer.")
          my_fmts = False
 
-      code  = "filename _tomodsx '"+self._sb.workpath+"_tomodsx' lrecl=1 recfm=f encoding=binary;\n"
+      code  = "filename _tomodsx '"+self._sb.workpath+"_tomodsx' recfm=v termstr=NL encoding='utf-8';\n"
       code += "data _null_; set "+tabname+self._sb._dsopts(dsopts)+";\n"
 
       if not my_fmts:
@@ -1848,7 +1736,7 @@ class SASsessionHTTP():
                if i % 10 == 0:
                   code +='\n'
 
-      code += "\nfile _tomodsx lrecl=1 recfm=f encoding=binary;\n"
+      code += "\nfile _tomodsx lrecl="+str(self.sascfg.lrecl)+" dlm="+cdelim+" recfm=v termstr=NL encoding='utf-8';\n"
       for i in range(nvars):
          if vartype[i] != 'FLOAT':
             code += "'"+varlist[i]+"'n = translate('"
@@ -1861,22 +1749,15 @@ class SASsessionHTTP():
                          ord(colsep.encode(self.sascfg.encoding))))
             if i % 10 == 0:
                code +='\n'
-      code += "\n"
-      last  = len(varlist)-1
+      code += "\nput "
       for i in range(nvars):
-         code += "put '"+varlist[i]+"'n "
-         if i != last:
-            code += cdelim+'; '
-         else:
-            code += rdelim+'; '
+         code += " '"+varlist[i]+"'n "
          if i % 10 == 0:
             code +='\n'
-      code += "run;"
+      code += rdelim+";\nrun;"
 
       ll = self.submit(code, "text")
 
-      ll = self.download(tmpcsv, self._sb.workpath+"_tomodsx")
-
       if k_dts is None:
          dts = {}
          for i in range(nvars):
@@ -1894,15 +1775,27 @@ class SASsessionHTTP():
 
       quoting = kwargs.pop('quoting', 3)
 
-      df = pd.read_csv(tmpcsv, index_col=idx_col, engine=eng, header=None, names=varlist, 
-                       sep=colsep, lineterminator=rowsep, dtype=dts, na_values=miss,
-                       encoding=self.sascfg.encoding, quoting=quoting, **kwargs)
+      code = "filename _sp_updn '"+self._sb.workpath+"_tomodsx' recfm=F encoding=binary lrecl=4096;"
 
-      if tmpdir:
-         tmpdir.cleanup()
-      else:
-         if not tempkeep:
-            os.remove(tmpcsv)
+      ll = self.submit(code, "text")
+      logf  = ll['LOG']
+
+      # GET data
+      conn = self.sascfg.HTTPConn; conn.connect()
+      headers={"Accept":"*/*","Content-Type":"application/octet-stream",
+               "Authorization":"Bearer "+self.sascfg._token}
+      conn.request('GET', self._uri_files+"/_sp_updn/content", headers=headers)
+      req = conn.getresponse()
+      status = req.status
+
+
+      sockout = _read_sock(req=req, method='DISK', rsep=(colsep+rowsep+'\n').encode(), rowsep=rowsep.encode())
+
+      df = pd.read_csv(sockout, index_col=idx_col, engine=eng, header=None, names=varlist, 
+                       sep=colsep, lineterminator=rowsep, dtype=dts, na_values=miss,
+                       encoding='utf-8', quoting=quoting, **kwargs)
+
+      conn.close()
 
       if k_dts is None:  # don't override these if user provided their own dtypes
          for i in range(nvars):
@@ -1910,4 +1803,45 @@ class SASsessionHTTP():
                if varcat[i] in self._sb.sas_date_fmts + self._sb.sas_time_fmts + self._sb.sas_datetime_fmts:
                   df[varlist[i]] = pd.to_datetime(df[varlist[i]], errors='coerce')
 
+      ll = self.submit("filename _sp_updn;", 'text')
+      logf += ll['LOG']
+      
       return df
+
+class _read_sock(io.StringIO):
+   def __init__(self, **kwargs):
+      self.req      = kwargs.get('req')
+      self.method   = kwargs.get('method', 'CSV')
+      self.rowsep   = kwargs.get('rowsep', b'\n')
+      self.rsep     = kwargs.get('rsep', self.rowsep)
+      self.datar    = b""
+
+   def read(self, size=4096):
+      datl    = 0
+      size    = max(size, 4096)
+      notarow = True
+
+      while datl < size or notarow:
+         data = self.req.read(size)
+         dl = len(data)
+   
+         if dl:
+            datl       += dl
+            self.datar += data
+            if notarow:
+               notarow = self.datar.count(self.rsep) <= 0
+         else:
+            if len(self.datar) <= 0:
+               return ''
+            else:
+               break
+   
+      data        = self.datar.rpartition(self.rsep)
+      if self.method == 'DISK':
+         datap    = (data[0]+data[1]).replace(self.rsep, self.rowsep)
+      else:
+         datap    = data[0]+data[1]
+      self.datar  = data[2]
+
+      return datap.decode()
+
